@@ -2,13 +2,15 @@
 ############prediction for "early", "parallel"############
 
 pred_lucid <- function(model,
-                          lucid_model = c("early", "parallel"),
-                          G,
-                          Z,
-                          Y = NULL,
-                          CoG = NULL,
-                          CoY = NULL,
-                          response = TRUE){
+                       lucid_model = c("early", "parallel"),
+                       G,
+                       Z,
+                       Y = NULL,
+                       CoG = NULL,
+                       CoY = NULL,
+                       response = TRUE,
+                       g_computation = FALSE,
+                       verbose = FALSE){
   ## 1.1 check data format ====
   if(is.null(G)) {
     stop("Input data 'G' is missing")
@@ -110,42 +112,98 @@ pred_lucid <- function(model,
     useY_flag <- ifelse(is.null(Y), FALSE, TRUE)
     # browser()
     # 1 - predict latent cluster
-    res <- Estep_early(beta = beta,
-                 mu = mu,
-                 sigma = Sigma.array,
-                 gamma = gamma,
-                 G = G,
-                 Z = Z,
-                 Y = Y,
-                 CoY = CoY,
-                 family.list = family.list,
-                 K = K,
-                 N = n,
-                 itr = 2,
-                 dimCoY = dimCoY,
-                 useY = useY_flag,
-                 ind.na = na_pattern$indicator_na)
+    if (g_computation == FALSE){
+      res <- Estep_early(beta = beta,
+                  mu = mu,
+                  sigma = Sigma.array,
+                  gamma = gamma,
+                  G = G,
+                  Z = Z,
+                  Y = Y,
+                  CoY = CoY,
+                  family.list = family.list,
+                  K = K,
+                  N = n,
+                  itr = 2,
+                  dimCoY = dimCoY,
+                  useY = useY_flag,
+                  ind.na = na_pattern$indicator_na)
+    }else{
+      res <- Estep_early_g(beta = beta,
+                         mu = mu,
+                         sigma = Sigma.array,
+                         gamma = gamma,
+                         G = G,
+                         Z = Z,
+                         Y = Y,
+                         CoY = CoY,
+                         family.list = family.list,
+                         K = K,
+                         N = n,
+                         itr = 2,
+                         dimCoY = dimCoY,
+                         useY = useY_flag,
+                         ind.na = na_pattern$indicator_na)
+    }
 
     # normalize the log-likelihood to probability
     res.r <- t(apply(res, 1, lse_vec))
     # predicted latent cluster
     pred.x <- sapply(1:n, function(x) return(nnet::which.is.max(res.r[x, ])))
-    mu_Y <- cbind(res.r, CoY) %*% gamma$beta
-
-
+    pred.x <- pred.x - 1
+    #here, since we modify the gamma, where it's just intecept + betas instead of mu_Y in the model output
+    #we need transfer gamma to mu_Y first 
+    model_mu_Y <- gamma$beta
+    model_mu_Y[2:K] <- model_mu_Y[1] + model_mu_Y[2:K]
+    mu_Y <- cbind(res.r, CoY) %*% model_mu_Y
+    #IP1 * beta0 + IP2 * (beta0 + beta1) + CoY*Gamma(CoY) is actually a weighed version of (Beta0 + beta1X)
+    
     if(model$family == "normal"){
+      #nomal Y is good
       pred.y <- mu_Y
     }
     if(model$family == "binary"){
+      #need to discuss here with Dave, for normal, we derive mu_Y for each cluster and weighted by IP, 
+      #for binary, do we get weight intercepts and betas and then map back to probability of the outcome??
+      #Before, Yinqi use weighted_mu_logit to map back, 
+      #For g_comp, how about keep logit of Y to estimate the causal effect of interest?
+      
+      #at first,I thought Yinqi is wrong
+      #But now I think it makes sense, for exmaple if IP2 > IP1
+      #then beta 0 is shrunk to 0
+      #then IP1 * beta0 + IP2 * (beta0 + beta1) is actually a weighed version of (Beta0 + beta1X)
+      
+      #now I think it's good
       pred.y <- exp(mu_Y) / (1 + exp(mu_Y))
       if(response == TRUE){
         pred.y <- as.numeric(pred.y > 0.5)
       }
     }
-
-    return(list(inclusion.p = res.r,
-                pred.x = pred.x,
-                pred.y = as.vector(pred.y)))
+    
+    if (g_computation == FALSE){
+      results <- list(inclusion.p = res.r,
+                      pred.x = pred.x,
+                      pred.y = as.vector(pred.y))
+    }else{
+      #compute the weighted (by pred.x and estimated mu from model) predicted mu for each obs of each feature
+      z = ncol(mu)
+      pred.z = matrix(NA, nrow = n, ncol = z)
+      colnames(pred.z) = names(mu)
+      for (i in 1:n){
+        p_i = res.r[i,]
+        mu_i = colSums(p_i * mu)
+        pred.z[i,] = mu_i
+      }
+      colnames(pred.z) = names(mu)
+      
+      results <- list(inclusion.p = res.r,
+                      pred.x = pred.x,
+                      pred.z = pred.z,
+                      pred.y = as.vector(pred.y))
+    }
+    
+    return(results)
+    
   }else if (match.arg(lucid_model) == "parallel"){
     ###LUCID in parallel#####
     if(is.null(colnames(G))){
@@ -207,14 +265,22 @@ pred_lucid <- function(model,
     Sigma = model$res_Sigma
     Gamma <- model$res_Gamma$Gamma
     useY_flag <- ifelse(is.null(Y), FALSE, TRUE)
+    
+    if(g_computation == FALSE){
+      Estep_array <- Estep(G = G, Z = Z, Y = Y,
+                           Beta = Beta, Mu = Mu, Sigma = Sigma, Delta = Gamma,
+                           family = model$family, useY = useY_flag, na_pattern = na_pattern)
 
-    Estep_array <- Estep(G = G, Z = Z, Y = Y,
-                         Beta = Beta, Mu = Mu, Sigma = Sigma, Delta = Gamma,
-                         family = model$family, useY = useY_flag, na_pattern = na_pattern)
+    }else{
+      Estep_array <- Estep_g(G = G, Z = Z, Y = Y,
+                           Beta = Beta, Mu = Mu, Sigma = Sigma, Delta = Gamma,
+                           family = model$family, useY = useY_flag, na_pattern = na_pattern)
+    }
+    
     Estep_r <- Estep_to_r(Estep_array = Estep_array,
                           K = K,
                           N = N)
-
+    
     post.p <- vector(mode = "list", length = nOmics)
     for(i in 1:nOmics) {
       post.p[[i]] = compute_res_r(r = Estep_r, N = N,layer = i)
@@ -222,17 +288,13 @@ pred_lucid <- function(model,
 
     # initialize container for predicted value
     pred_X <- vector(mode = "list", length = nOmics)
-    pred_z <- vector(mode = "list", length = nOmics)
     # prediction of X and Y based on fitted data
 
 
       # 1 - prediction for X
       for (i in 1:nOmics) {
-        r_margin <- t(sapply(1:N, function(j) {
-          marginSums(lastInd(Estep_r,j), margin = i)
-        }))
-        pred_X[[i]] <- map(r_margin)
-        pred_z[[i]] <- r_margin
+        pred_X[[i]] <- sapply(1:N, function(x) return(nnet::which.is.max(post.p[[i]][x, ])))
+        pred_X[[i]] <- pred_X[[i]] - 1
       }
 
     # 2 - prediction for Y
@@ -244,7 +306,7 @@ pred_lucid <- function(model,
         marginSums(lastInd(r,j), margin = i)
       }))
       r_fit <- as.data.frame(r_matrix[, -c(1, K[1] + 1)])
-
+      #Here ip are used to get the weighted Y, we remove reference ip columns as if it's all categorical predicors.
       if(model$family == "gaussian") {
         if(is.null(CoY)) {
           pred_Y <- predict(model$res_Gamma$fit, newdata = r_fit)
@@ -380,10 +442,32 @@ pred_lucid <- function(model,
           }else{pred_Y <- predict(model$res_Gamma$fit, newdata = r_x, type = "response")}
         }
       }}
-
-      return(list(inclusion.p = post.p,
-                  pred.x = pred_X,
-                  pred.y = pred_Y))
+    
+    if (g_computation == FALSE){
+      results <- list(inclusion.p = post.p,
+                      pred.x = pred_X,
+                      pred.y = pred_Y)
+    }else{
+      #compute the weighted (by pred.x and estimated mu from model) predicted mu for each obs of each feature
+      pred.z <- vector(mode = "list", length = nOmics)
+      for (i in 1:nOmics){
+        mu_i = t(Mu[[i]])
+        z = ncol(mu_i)
+        pred_layer_z = matrix(NA, nrow = N, ncol = z)
+        colnames(pred_layer_z) = names(mu_i)
+        for (j in 1:N){
+          p_j = post.p[[i]][j,]
+          mu_j = colSums(p_j * mu_i)
+          pred_layer_z[j,] = mu_j
+        }
+        pred.z[[i]] <- pred_layer_z
+      }
+      results <- list(inclusion.p = post.p,
+                      pred.x = pred_X,
+                      pred.z = pred.z,
+                      pred.y = pred_Y)
+    }
+      return(results)
     }
   }
 
