@@ -60,19 +60,19 @@
 #'
 #' # find the optimal model over the grid of K
 #' tune_K <- tune_lucid(G = G, Z = Z, Y = Y_normal, lucid_model = "early",
-#'  useY = FALSE, tol = 1e-3,
-#' seed = 1, K = 2:5)
+#'  useY = FALSE, tol = 1e-2,
+#' seed = 1, K = 2:3)
 #'
 #' # tune penalties
 #' tune_Rho_G <- tune_lucid(G = G, Z = Z, Y = Y_normal, lucid_model = "early",
-#'  useY = FALSE, tol = 1e-3,
-#' seed = 1, K = 2, Rho_G = c(0.1, 0.2, 0.3, 0.4))
+#'  useY = FALSE, tol = 1e-2,
+#' seed = 1, K = 2, Rho_G = c(0.1, 0.2))
 #' tune_Rho_Z_Mu <- tune_lucid(G = G, Z = Z, Y = Y_normal, lucid_model = "early", 
-#' useY = FALSE, tol = 1e-3,
-#' seed = 1, K = 2, Rho_Z_Mu = c(10, 20, 30, 40))
+#' useY = FALSE, tol = 1e-2,
+#' seed = 1, K = 2, Rho_Z_Mu = c(10, 20))
 #' tune_Rho_Z_Cov <- tune_lucid(G = G, Z = Z, Y = Y_normal, lucid_model = "early", 
-#' useY = FALSE, tol = 1e-3,
-#' seed = 1, K = 2, Rho_Z_Cov = c(0.1, 0.2, 0.3))
+#' useY = FALSE, tol = 1e-2,
+#' seed = 1, K = 2, Rho_Z_Cov = c(0.1, 0.2))
 #'}
 #'
 tune_lucid <- function(G,
@@ -210,7 +210,7 @@ tune_lucid_auxi <- function(G,
     if("try-error" %in% class(fit)) {
       tune_list[i, 5] <- NA
     } else {
-      tune_list[i, 5] <- summary_lucid(fit)$BIC
+      tune_list[i, 5] <- summary(fit)$BIC
     }
     res_model[[i]] <- fit
   }
